package com.muyu.service;

import com.muyu.common.PageList;
import com.muyu.domain.Vehicle;
import com.muyu.domain.req.CheckPositionReq;
import com.muyu.domain.req.GearReq;
import com.muyu.domain.req.MsgReq;
import com.muyu.domain.req.VehicleInstanceListReq;
import com.muyu.domain.resp.VehicleInstanceResp;

/**
 * @author DongZeLiang
 * @version 1.0
 * @description 车辆实例业务层
 * @date 2023/11/22
 */
public interface VehicleInstanceService {

    /**
     * 根据车辆生成车辆实例
     * @param vehicle 车辆信息
     */
    public void init(Vehicle vehicle);


    /**
     * 车辆结果对象
     *
     * @param vehicleInstanceListReq 车辆查询
     *
     * @return 车辆对象
     */
    PageList<VehicleInstanceResp> queryList (VehicleInstanceListReq vehicleInstanceListReq);

    /**
     * 车辆客户端初始化
     * @param vin vin
     */
    void vehicleClientInit (String vin);

    /**
     * 车辆客户端关闭
     * @param vin vin
     */
    void vehicleClientClose (String vin);

    /**
     * 选择车辆轨迹
     * @param checkPositionReq 切换轨迹
     */
    void checkPosition (CheckPositionReq checkPositionReq);

    /**
     * 车辆消息操作
     * @param msgReq
     */
    void msg (MsgReq msgReq);

    /**
     * 设置档位
     * @param gearReq
     */
    void gear (GearReq gearReq);

    /**
     * 修改车辆报文
     * @param vin vin
     * @param statusKey 状态键
     * @param statusValue 状态值
     */
    void editStatus (String vin, String statusKey, Integer statusValue);


    /**
     * 一键上线
     */
    public void unifiedOnline();

    /**
     * 一键离线
     */
    public void unifiedOffline();

    /**
     * 一键上报
     */
    public void unifiedSend();

    /**
     * 一键重置路径
     */
    void unifiedPosition ();

    /**
     * 一键取消上报
     */
    public void unifiedStop();

}
